% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KO_calibration.R
\name{KO_calibration}
\alias{KO_calibration}
\title{The Kennedy and O'Hagan (KO) approach to computer model calibration}
\usage{
KO_calibration(Y, X, y_s, theta_min, theta_max, sig, tau, thinning = 10,
  alpha = 5/2, range = 1/8, range_fix = TRUE, Burn_in = 1000,
  n_mc = 1000, jump, hyper_KO)
}
\arguments{
\item{Y}{The (noisy) repsonse from the physical system.}

\item{X}{The design points corresponding to the physical responses.}

\item{y_s}{The computer model y_s(x, theta), where x is a sequence of design points and.
theta is a q-dimensional vector of the calibration parameter.}

\item{theta_min}{A q-dimensional vector specifying the lower bound for the calibration parameter.}

\item{theta_max}{A q-dimensional vector specifying the upper bound for the calibration parameter.}

\item{sig}{The standard deviation of the noises.}

\item{tau}{The scaling factor of the covariance function.}

\item{thinning}{The thinning size for the MCMC samples. Default value is 10.}

\item{alpha}{The roughness parameter for the Matern covariance function. Default value is 5/2.}

\item{range}{The range parameter for the Matern covariance function. Default value is 1/8.}

\item{range_fix}{A logical value to decide whether the range parameter is sampled within MCMC.}

\item{Burn_in}{The number of burn-in iterations for the MCMC. Default value is 1000.}

\item{n_mc}{The number of posterior samples to be collected. Default value is 1000.}

\item{jump}{The size of random walk for the Metropolis-Hastings method.}

\item{hyper_KO}{A list containing a_psi (shape) and b_psi (rate) for the inverse-Gamma hyperprior for the range parameter.}
}
\value{
A n_mc-by-q matrix where the rows are the posterior samples of theta.
}
\description{
Draw posterior samples based on the KO calibration approach using the
Metroplis-Hastings MCMC method. The discrepancy function is imposed a Gaussian process
prior with roughness parameter alpha, 0 < alpha <= infinity.When alpha = infinity,
the covariance function is set to the squared-exponential covariance function.
}
\examples{
#######################################
# Simulate data for physical system   #
#######################################
library(lhs)
theta0 = c(0.2, 0.3)
theta_min = c(0, 0)
theta_max = c(0.25, 0.5)
y_s = function(x, theta){
  return(7 * (sin(2 * pi * theta[1] - pi))^2 +
           2 * (2 * pi * theta[2] - pi)^2 * sin(2 * pi * x - pi))
}
X = maximinLHS(50, 1)
n = nrow(X)
sig = 0.2
Y = y_s(X, theta0) + rnorm(nrow(X), mean = 0, sd = sig)
#######################################
# Apply the KO calibration approach   #
#######################################
tau = 5
hyper_KO = NULL
hyper_KO$a_psi = hyper_KO$b_psi = 2
theta_KO_mc = KO_calibration(Y, X, y_s, theta_min, theta_max, sig, tau, alpha = 5/2,
                             range_fix = FALSE, jump = c(0.1, 0.01), hyper_KO = hyper_KO)
n_mc = nrow(theta_KO_mc)
x_seq = seq(0, 1, length = 500)
ys_KO = matrix(NA, nrow = n_mc, ncol = length(x_seq))
for (i in 1:n_mc){
  ys_KO[i, ] = y_s(x_seq, theta_KO_mc[i, ])
}
#######################################
# Plot the calibrated computer model   #
#######################################
plot(x_seq, y_s(x_seq, theta0), type = "l",
     lwd = 1, xlab = "x", ylab = expression(eta),
     ylim = c(1, 11),
     main = "(d) KO Calibration")
points(X, Y, type = "p", pch = 1)
points(x_seq, y_s(x_seq, apply(theta_KO_mc, 2, mean)), type = "l", lty = "dashed")
polygon(c(x_seq, rev(x_seq)),
        c(apply(ys_KO, 2, quantile, 0.975), rev(apply(ys_KO, 2, quantile, 0.025))),
        col = adjustcolor("grey50",alpha.f=0.5), border = FALSE)
legend("topleft",
       legend = c("Observation", "True Process", "KO Calibrated Model (mean)"), lwd = c(1, 1, 1),
       pch = c(1, NA, NA), lty = c(NA, "solid", "dashed"), bty = "n")
}
\references{
Kennedy M C, O'Hagan A. Bayesian calibration of computer models[J]. Journal of the Royal Statistical Society: Series B
(Statistical Methodology), 2001, 63(3): 425-464.

Plumlee M. Bayesian calibration of inexact computer models[J]. Journal of the American Statistical Association,
2017, 112(519): 1274-1285.MLA
}
