% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L2_calibration.R
\name{L2_calibration}
\alias{L2_calibration}
\title{The L2-calibration method for calibrating computer model}
\usage{
L2_calibration(Y, X, y_s, Dy_s, theta_min, theta_max, lambda_seq = exp(seq(-6,
  -1, by = 1) * log(10)), alpha = 5/2, range = 1/8, N = 2000,
  N_multi = 20)
}
\arguments{
\item{Y}{The (noisy) repsonse from the physical system.}

\item{X}{The design points corresponding to the physical responses.}

\item{y_s}{The computer model y_s(x, theta), where x is a sequence of design points and.
theta is a q-dimensional vector of the calibration parameter.}

\item{Dy_s}{The derivative of y_s(x, theta), where x is a sequence of design points and.
theta is a q-dimensional vector of the calibration parameter.}

\item{theta_min}{A q-dimensional vector specifying the lower bound for the calibration parameter.}

\item{theta_max}{A q-dimensional vector specifying the upper bound for the calibration parameter.}

\item{lambda_seq}{A sequence of tunning parametes lambda. The optimal value of lambda is selected using
leave-one-out cross validation (LOOCV).}

\item{alpha}{The roughness parameter for the Matern kernel function. Default value is 5/2.}

\item{range}{The range parameter for the Matern kernel function. Default value is 1/8.}

\item{N}{The number of samples for AdaGrad. Default value is 2000.}

\item{N_multi}{The number of trials for running AdaGrad multiple times to search for the global optimum.}
}
\value{
A list containing:

x_seq: A grid_size-by-q matrix of design points where the calibrated computer model and the estimated
physical system are evaluated;

eta_hat: A grid_size-dimensional vector of the estimated physical system evaluated at x_seq;

theta_L2: A q-dimensional vector of the estimated calibration parameter.
}
\description{
Provides an estimates to the calibration parameter based on the physical
data and the computer model using the L2-calibration method and the stochastic gradient descent algorithm with adaptive stepsize (AdaGrad). The physical system is
estimated using kernel ridge regression with the Matern kernel with roughness parameter alpha,
0 < alpha <= infinity. When alpha = infinity, the kernel function is set to the squared-exponential
kernel (Gaussian kernel).
}
\examples{
#######################################
# Simulate data for physical system   #
#######################################
library(lhs)
theta0 = c(0.2, 0.3)
theta_min = c(0, 0)
theta_max = c(0.25, 0.5)
y_s = function(x, theta){
  return(7 * (sin(2 * pi * theta[1] - pi))^2 +
           2 * (2 * pi * theta[2] - pi)^2 * sin(2 * pi * x - pi))
}
Dy_s = function(x, theta){
  n = nrow(x)
  Dy_s1 = 14 * sin(2 * pi * theta[1] - pi) * cos(2 * pi * theta[1] - pi) * (2 * pi)
  Dy_s2 = 4 * (2 * pi * theta[2] - pi) * (2 * pi) * sin(2 * pi * x - pi)
  Dys = matrix(NA, nrow = 2, ncol = n)
  Dys[1, ] = Dy_s1
  Dys[2, ] = Dy_s2
  return(Dys)
}
X = maximinLHS(50, 1)
n = nrow(X)
sig = 0.2
Y = y_s(X, theta0) + rnorm(nrow(X), mean = 0, sd = sig)
#######################################
# Apply the L2 calibration approach   #
#######################################
L2_calib = L2_calibration(Y, X, y_s, Dy_s, theta_min, theta_max, alpha = 3/2, range = 1/8)
x = sort(L2_calib$x, decreasing = FALSE, index.return = TRUE)
x_seq = x$x
eta_hat = L2_calib$eta_hat[x$ix]
theta_hat = L2_calib$theta_L2
#######################################
# Plot the calibrated computer model  #
#######################################
plot(X, Y, type = "p", pch = 1, xlab = "x", ylab = expression(eta),
     main = expression(paste("Kernel Ridge Regression for ", eta, sep = "")))
points(x_seq, y_s(x_seq, theta0), type = "l", lty = "dashed", lwd = 1)
points(x_seq, y_s(x_seq, theta_hat), type = "l", lwd = 1, col = "red")
legend("topleft",
       legend = c("Observation", "Calibrated Computer Model", "True Process"),
       lwd = c(1, 1, 1), pch = c(1, NA, NA), col = c("black", "red", "black"),
       lty = c(NA, "solid", "dashed"), bty = "n")
}
\references{
Tuo R, Wu C F J. Efficient calibration for imperfect computer models[J]. The Annals of Statistics, 2015, 43(6): 2331-2352.
}
