% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proj_calibration.R
\name{proj_calibration}
\alias{proj_calibration}
\title{Bayesian projected calibration for computer models}
\usage{
proj_calibration(Y, X, y_s, theta_min, theta_max, sig, tau, alpha = 5/2,
  range = 1/8, n_mc = 1000, grid_size = 500)
}
\arguments{
\item{Y}{The (noisy) repsonse from the physical system.}

\item{X}{The design points corresponding to the physical responses.}

\item{y_s}{The computer model y_s(x, theta), where x is a sequence of design points and.
theta is a q-dimensional vector of the calibration parameter.}

\item{theta_min}{A q-dimensional vector specifying the lower bound for the calibration parameter.}

\item{theta_max}{A q-dimensional vector specifying the upper bound for the calibration parameter.}

\item{sig}{The standard deviation of the noises.}

\item{tau}{The scaling factor of the covariance function.}

\item{alpha}{The roughness parameter for the Matern covariance function. Default value is 5/2.}

\item{range}{The range parameter for the Matern covariance function. Default value is 1/8.}

\item{n_mc}{The number of posterior samples to be collected. Default value is 1000.}

\item{grid_size}{The number of points for discretization for numerical integration. Default value is 500.}
}
\value{
A list containing:

x_seq: A grid_size-by-q matrix of design points where the calibrated computer model and the estimated
physical system are evaluated;

eta_mean: A grid_size-dimensional vector the posterior mean of the physical system evaluated at x_seq;

eta_cov: A grid_size-by-grid_size matrix of the posterior covariance function evaluated at x_seq;

theta_star_mc: A n_mc-by-q matrix where the rows are the posterior samples of the calibration parameter.
}
\description{
Draw posterior samples based on the Bayesian projected calibration approach.
The physical system is imposed a Gaussian process prior with roughness parameter alpha,
0 < alpha <= infinity.When alpha = infinity, the covariance function is set to the
squared-exponential covariance function.
}
\examples{
#######################################
# Simulate data for physical system   #
#######################################
library(lhs)
theta0 = c(0.2, 0.3)
theta_min = c(0, 0)
theta_max = c(0.25, 0.5)
y_s = function(x, theta){
  return(7 * (sin(2 * pi * theta[1] - pi))^2 +
           2 * (2 * pi * theta[2] - pi)^2 * sin(2 * pi * x - pi))
}
X = maximinLHS(50, 1)
n = nrow(X)
sig = 0.2
Y = y_s(X, theta0) + rnorm(nrow(X), mean = 0, sd = sig)
############################################
# Apply the Bayesian projected calibration #
############################################
tau = 5
proj_calib = proj_calibration(Y, X, y_s, theta_min, theta_max, sig, tau, alpha = 5/2, range = 1/8)
x = sort(proj_calib$x, decreasing = FALSE, index.return = TRUE)
x_seq = x$x
eta_mean = proj_calib$eta_mean[x$ix]
eta_cov = proj_calib$eta_cov[x$ix, x$ix]
theta_star_mc = proj_calib$theta_star_mc
#######################################
# Plot the calibrated computer model   #
#######################################
plot(X, Y, type = "p", pch = 1,
     lwd = 2, xlab = "x", ylab = expression(eta),
     main = expression(paste("Posterior Samples of ", eta)))
points(x_seq, y_s(x_seq, theta0), type = "l", lty = "dashed", lwd = 1)
points(x_seq, y_s(x_seq, apply(theta_star_mc, 2, mean)),
       type = "l", lty = "dotted", lwd = 1, col = "blue")
points(x_seq, eta_mean, type = "l", lwd = 1, col = "red")
polygon(c(x_seq, rev(x_seq)),
        c(eta_mean  + sqrt(diag(eta_cov)) * qnorm(0.95),
          rev(eta_mean + sqrt(diag(eta_cov)) * qnorm(0.05))),
        col = adjustcolor("grey50",alpha.f=0.5), border = FALSE)
legend("topleft", legend = c("Observation", "GP Posterior Mean",
                             "True Process", "Calibrated Computer Model"),
       lwd = c(1, 1, 1, 1), pch = c(1, NA, NA, NA),
       lty = c(NA, "solid", "dashed", "dotted"),
       col = c("black", "red", "black", "blue"),
       bty = "n")
}
